/*
 * 	pam_tcfs.so v1.0
 *	
 *	written by Luigi Catuogno [luicat@mikonos.dia.unisa.it]
 *
 *	Jun 30 1997
 */

#define PAM_SM_AUTH
#define PAM_SM_SESSION
#define PAM_SM_PASSWORD
#include <security/pam_modules.h>
#include <syslog.h>

#include "tcfspasswd.h"


/* PAM--- authentication management functions --- */

PAM_EXTERN int 
pam_sm_authenticate (pam_handle_t * pamh, int flags, int argc
		     ,const char **argv)
{
      int r,retval,dummy;
      char *p,*u;
      static char *tk;
      static struct tcfspasswd t;

      r=pam_get_item(pamh,PAM_USER,&u);
      if(r!=PAM_SUCCESS)
	{
	 syslog(LOG_WARNING,"pam_tcfs got problems retrieving user name");
	 return r;
	}

      r=pam_get_item(pamh,PAM_AUTHTOK,&p);
      if(r!=PAM_SUCCESS || (!p))
	{
	 syslog(LOG_WARNING,"pam_tcfs got problems retrieving auth. token");
	 return r;
	}
	
      
      if(!tcfs_getpwnam(u,&t))
	{
	 syslog(LOG_INFO,"%s is not a tcfs user.",u);
	 return PAM_AUTH_ERR;
	}
      
      tk=(char*)malloc(KEYSIZE);
      if(!tk)
	{	
	 syslog(LOG_WARNING,"malloc failed");
	 exit(1);
	}
      
      tcfs_key(u,p,t.upw,&tk);
    
      retval=pam_set_data(pamh,"tcfskey",tk,0);
      if(retval!=PAM_SUCCESS)
	 return PAM_ABORT;

      syslog(LOG_INFO,"%s sucessfully attached tcfs",u);
      return PAM_SUCCESS;
	
}

PAM_EXTERN int 
pam_sm_setcred (pam_handle_t * pamh, int flags, int argc,
		const char **argv)
{
  return PAM_SUCCESS;
}

PAM_EXTERN int 
pam_sm_chauthtok (pam_handle_t * pamh, int flags, int argc
		  ,const char **argv)
{
  char *p;
  char *u, *o;
  int r;


  if (flags&PAM_PRELIM_CHECK)
	if(!ex_lockfile())
		return PAM_AUTHTOK_LOCK_BUSY;
	else
		return PAM_SUCCESS;
  

  r = pam_get_item (pamh, PAM_USER, &u);
  if (r != PAM_SUCCESS)
    return r;

  r = pam_get_item (pamh, PAM_AUTHTOK, &p);
  if (r != PAM_SUCCESS)
    return r;

  r = pam_get_item (pamh, PAM_OLDAUTHTOK, &o);
  if (r != PAM_SUCCESS)
    return r;

  if (!(*u)||!(*o)||!(*p))
	{
	 return PAM_AUTHTOK_ERR;
	}

  if (tcfs_chgkey (u, o, p))
    return PAM_SUCCESS;


  return PAM_AUTHTOK_ERR;
}

/* --- session management --- */

PAM_EXTERN int 
pam_sm_open_session (pam_handle_t * pamh, int flags, int argc
		     ,const char **argv)
{
  char *p, *u;
  static *pw, *un;
  int retval;

  retval = pam_get_item (pamh, PAM_USER, (void *) &u);
  if (retval != PAM_SUCCESS || (!u))
    {
      return retval;
    }
	
  
  retval = pam_get_data (pamh,"tcfskey", (void *) &p);
  if (retval != PAM_SUCCESS)
      return retval;

  if (!p)
	{
	 syslog(LOG_WARNING,"pam_tcfs got problems retrieving auth. token");
	 return PAM_SESSION_ERR;
	}


  retval = tcfs_enable(u,p);
  if (retval)
    {
      syslog(LOG_INFO,"starting tcfs session user %s",u);
      return PAM_SUCCESS;
    }
  return PAM_SESSION_ERR;
}

PAM_EXTERN int 
pam_sm_close_session (pam_handle_t * pamh, int flags, int argc
		      ,const char **argv)
{
  char *user_name, *k;
  int retval;

  retval = pam_get_data (pamh,"tcfskey",&k);
  if (retval!=PAM_SUCCESS)
	return retval;

  retval = pam_get_item (pamh, PAM_USER, (void *) &user_name);
  if (retval != PAM_SUCCESS)
      return retval;

  retval = tcfs_disable (user_name);
  if (retval)
    {
      syslog(LOG_INFO,"ending tcfs session user %s",user_name);
      return PAM_SUCCESS;
    }

  return PAM_SESSION_ERR;
}

/* end of module definition */

/* static module data */
#ifdef PAM_STATIC
struct pam_module _pam_tcfs_modstruct =
{
  "pam_tcfs",
  pam_sm_authenticate,
  NULL,
  NULL,
  pam_sm_open_session,
  pam_sm_close_session,
  pam_sm_chauthtok
};
#endif
